<?php

namespace Drupal\tmgmt_openai;

use Drupal\tmgmt\TranslatorPluginUiBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * OpenAI translator UI.
 */
class OpenAiTranslatorUi extends TranslatorPluginUiBase {

  /**
   * OpenAI completion models.
   */
  protected $models = [
    "GPT-3" => [
      "text-davinci-003" => "text-davinci-003",
      "text-davinci-002" => "text-davinci-002",
      "text-davinci-001" =>"text-davinci-001",
      "text-curie-001" => "text-curie-001",
      "text-ada-001" => "text-ada-001",
      "text-babbage-001" => "text-babbage-001",
      "davinci" => "davinci",
      "davinci-instruct-beta" => "davinci-instruct-beta",
      "curie" => "curie",
      "curie-instruct-beta" => "curie-instruct-beta",
      "babbage" => "babbage",
      "ada" => "ada",
    ]
  ];

  /**
   * Overrides TMGMTDefaultTranslatorUIController::pluginSettingsForm().
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\tmgmt\TranslatorInterface $translator */
    $translator = $form_state->getFormObject()->getEntity();
    $settings = $translator->getSettings();
    $form['openai_api_service'] = array(
      '#type' => 'select',
      '#title' => $this->t('Service Provider'),
      '#required' => TRUE,
      '#options' => [
        'openai' => $this->t('OpenAI'),
        'azure_openai' => $this->t('Microsoft Azure OpenAI'),
      ],
      '#default_value' => !empty($settings['openai_api_service']) ? $settings['openai_api_service'] : 'openai',
    );
    $form['openai_api_key'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#default_value' => !empty($settings['openai_api_key']) ? $settings['openai_api_key'] : '',
      '#description' => $this->t('Obtain your API Key from <a href="@link" target="_blank">here</a>', ['@link' => 'https://platform.openai.com/account/api-keys']),
      '#required' => TRUE,
    );
    $form['openai_api_org'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('API Organization'),
      '#default_value' => !empty($settings['openai_api_org']) ? $settings['openai_api_org'] : '',
      '#states' => [
        'visible' => [
          ':input[name="settings[openai_api_service]"]' => ['value' => 'openai'],
        ],
      ],
    );
    $form['openai_api_base_url'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Azure OpenAI Base URL'),
      '#default_value' => !empty($settings['openai_api_base_url']) ? $settings['openai_api_base_url'] : '',
      '#description' => $this->t('Format example: {your-resource-name}.openai.azure.com/openai/deployments/{deployment-id}'),
      '#states' => [
        'visible' => [
          ':input[name="settings[openai_api_service]"]' => ['value' => 'azure_openai'],
        ],
      ],
    );
    $form['openai_api_version'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Azure OpenAI Version'),
      '#default_value' => !empty($settings['openai_api_version']) ? $settings['openai_api_version'] : '',
      '#states' => [
        'visible' => [
          ':input[name="settings[openai_api_service]"]' => ['value' => 'azure_openai'],
        ],
      ],
    );
    $form['openai'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced'),
    ];
    $form['openai']['prompt'] = array(
      '#type' => 'textarea',
      '#title' => $this->t('Prompt'),
      '#default_value' => !empty($settings['openai']['prompt']) ? $settings['openai']['prompt'] : "Translate from %source% into %target% language\n\n%text%",
      '#description' => $this->t('You can adjust tone of the translated text and/or adjust style of the text. Supported tokens: <strong>%source%</strong> - source language (translate from), <strong>%target%</strong> - target language (translate to), <strong>%text%</strong> - text to translate.'),
      '#placeholder' => "Translate from %source% into %target% language\n\n%text%",
      '#required' => TRUE,
    );
    $form['openai']['model'] = [
      '#type' => 'select',
      '#title' => $this->t('Model'),
      '#description' => $this->t('The model which will generate the completion. Some models are suitable for natural language tasks, others specialize in code. <a href="@link" target="_blank">Learn more</a>.', ['@link' => 'https://platform.openai.com/docs/models']),
      '#options' => $this->models,
      '#default_value' => !empty($settings['openai']['model']) ? $settings['openai']['model'] : 'text-davinci-003',
      '#required' => TRUE,
    ];
    $form['openai']['temperature'] = [
      '#type' => 'number',
      '#title' => $this->t('Temperature'),
      '#description' => $this->t('Controls randomness: Lowering results in less random completions. As the temperature approaches zero, the model will become deterministic and repetitive.'),
      '#min' => 0,
      '#max' => 2,
      '#step' => 0.01,
      '#default_value' => !empty($settings['openai']['temperature']) ? $settings['openai']['temperature'] : 0.7,
    ];
    $form['openai']['max_tokens'] = [
      '#type' => 'number',
      '#title' => $this->t('Maximum length'),
      '#description' => $this->t('The maximum number of tokens to generate. Requests can use up to 2,048 or 4,000 tokens shared between prompt and completion. The exact limit varies by model. (One token is roughly 4 characters for normal English text)'),
      '#min' => 1,
      '#max' => 4000,
      '#default_value' => !empty($settings['openai']['max_tokens']) ? $settings['openai']['max_tokens'] : 4000,
    ];
    $form['openai']['top_p'] = [
      '#type' => 'number',
      '#title' => $this->t('Top P'),
      '#description' => $this->t('Controls diversity via nucleus sampling: 0.5 means half of all likelihood-weighted options are considered.'),
      '#min' => 0,
      '#max' => 1,
      '#step' => 0.01,
      '#default_value' => !empty($settings['openai']['top_p']) ? $settings['openai']['top_p'] : 1.0,
    ];
    $form['openai']['frequency_penalty'] = [
      '#type' => 'number',
      '#title' => $this->t('Frequency penalty'),
      '#description' => t("How much to penalize new tokens based on their existing frequency in the text so far. Decreases the model's likelihood to repeat the same line verbatim."),
      '#min' => 0,
      '#max' => 2,
      '#step' => 0.01,
      '#default_value' => !empty($settings['openai']['frequency_penalty']) ? $settings['openai']['frequency_penalty'] : 0.0,
    ];
    $form['openai']['presence_penalty'] = [
      '#type' => 'number',
      '#title' => $this->t('Presence penalty'),
      '#description' => t("How much to penalize new tokens based on whether they appear in the text so far. Increases the model's likelihood to talk about new topics."),
      '#min' => 0,
      '#max' => 2,
      '#step' => 0.01,
      '#default_value' => !empty($settings['openai']['presence_penalty']) ? $settings['openai']['presence_penalty'] : 1,
    ];
    $form += parent::addConnectButton();
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);
    /** @var \Drupal\tmgmt\TranslatorInterface $translator */
    $translator = $form_state->getFormObject()->getEntity();
    if (!empty($form_state->getValue('openai_api_key'))) {
      $connected = $translator->getPlugin()->openAiClient($translator)->getModels();
      if (empty($connected)) {
        $form_state->setErrorByName('settings][openai_api_key', $this->t('The "OpenAI API key" is not correct.'));
      }
    }
  }

}
