<?php

namespace Drupal\tmgmt_openai\Plugin\tmgmt\Translator;

use Drupal\Component\Utility\Html;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\tmgmt\ContinuousTranslatorInterface;
use Drupal\tmgmt\Data;
use Drupal\tmgmt\Entity\Job;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\TranslatorInterface;
use Drupal\tmgmt\Entity\Translator;
use Drupal\tmgmt\TMGMTException;
use Drupal\tmgmt\TranslatorPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use \Drupal\tmgmt\Translator\AvailableResult;

/**
 * OpenAI translator plugin.
 *
 * @TranslatorPlugin(
 *   id = "openai",
 *   label = @Translation("OpenAI"),
 *   description = @Translation("OpenAI Translator service."),
 *   ui = "Drupal\tmgmt_openai\OpenAiTranslatorUi",
 *   logo = "icons/openai.svg",
 * )
 */
class OpenAiTranslator extends TranslatorPluginBase implements ContainerFactoryPluginInterface, ContinuousTranslatorInterface {

  /**
   * OpenAI client.
   *
   * @var \OpenAI
   */
  protected $client;

  /**
   * Data help service.
   *
   * @var \Drupal\tmgmt\Data
   */
  protected $dataHelper;

    /**
   * Constructs a LocalActionBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\tmgmt\Data $data_helper
   *   Data helper service.
   */
  public function __construct(array $configuration, string $plugin_id, array $plugin_definition, Data $data_helper) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->dataHelper = $data_helper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('tmgmt.data')
    );
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::checkAvailable().
   */
  public function checkAvailable(TranslatorInterface $translator) {
    if ($translator->getSetting('openai_api_key')) {
      return AvailableResult::yes();
    }

    return AvailableResult::no(t('@translator is not available. Make sure it is properly <a href=:configured>configured</a>.', [
      '@translator' => $translator->label(),
      ':configured' => $translator->toUrl()->toString()
      ]));
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::requestTranslation().
   */
  public function requestTranslation(JobInterface $job) {
    $this->requestJobItemsTranslation($job->getItems());
    if (!$job->isRejected()) {
      $job->submitted('The translation job has been submitted.');
    }
  }

  /**
   * Helper method to do translation request.
   *
   * @param Job $job
   *   TMGMT Job to be used for translation.
   * @param array|string $q
   *   Text/texts to be translated.
   *
   * @return array
   *   Userialized JSON containing translated texts.
   */
  protected function openAiRequestTranslation(Job $job, $q) {
    return $this->doRequest($job->getTranslator(), 'translate', [
      'source' => $job->getSourceLangcode(),
      'target' => $job->getTargetLangcode(),
      'text' => $q,
    ]);
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::getSupportedRemoteLanguages().
   */
  public function getSupportedRemoteLanguages(TranslatorInterface $translator) {
    $languages = [];
    $site_languages = \Drupal::languageManager()->getLanguages();
    foreach ($site_languages as $langcode => $language) {
      $languages[$langcode] = $language->getName();
    }
    return $languages;
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::getSupportedTargetLanguages().
   */
  public function getSupportedTargetLanguages(TranslatorInterface $translator, $source_language) {
    $languages = $this->getSupportedRemoteLanguages($translator);
    // There are no language pairs, any supported language can be translated
    // into the others. If the source language is part of the languages,
    // then return them all, just remove the source language.
    if (array_key_exists($source_language, $languages)) {
      unset($languages[$source_language]);
      return $languages;
    }
    return array();
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::hasCheckoutSettings().
   */
  public function hasCheckoutSettings(JobInterface $job) {
    return FALSE;
  }

  /**
   * Local method to do request to OpenAI service.
   *
   * @param Translator $translator
   *   The translator entity to get the settings from.
   * @param string $action
   *   Action to be performed [translate, languages, detect]
   * @param array $request_query
   *   (Optional) Additional query params to be passed into the request.
   * @param array $options
   *   (Optional) Additional options that will be passed into drupal_http_request().
   *
   * @return array object
   *   Unserialized JSON response from Google.
   *
   * @throws TMGMTException
   *   - Invalid action provided
   *   - Unable to connect to the Google Service
   *   - Error returned by the Google Service
   */
  protected function doRequest(Translator $translator, $action, array $request_query = array(), array $options = array()) {
    if (!in_array($action, ['translate', 'languages'])) {
      throw new TMGMTException('Invalid action requested: @action', array('@action' => $action));
    }

    // Send the request with the query.
    try {
      $settings = $translator->getSettings();
      $site_languages = \Drupal::languageManager()->getLanguages();
      $client = $this->openAiClient($translator);
      $prompt = $settings['openai']['prompt'];
      if (empty($prompt)) {
        $prompt = 'Translate from %source% into %target% language\n\n%text%';
      }
      $configuration = [
        'prompt' => str_replace(
          ['%source%', '%target%', '%text%'],
          [$site_languages[$request_query['source']]->getName(), $site_languages[$request_query['target']]->getName(), $request_query['text']],
          $prompt
        ), 
        'model' => (string) $settings['openai']['model'],
        'temperature' => (float) $settings['openai']['temperature'],
        'max_tokens' => (int) $settings['openai']['max_tokens'],
        'top_p' => (float) $settings['openai']['top_p'],
        'frequency_penalty' => (float) $settings['openai']['frequency_penalty'],
        'presence_penalty' => (float) $settings['openai']['presence_penalty'],
      ];
      $response = $client->completionsRequest($configuration);
    }
    catch (\Exception $e) {
      throw new TMGMTException('OpenAI service returned following error: @error', ['@error' => $e->getMessage()]);
    }
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function requestJobItemsTranslation(array $job_items) {
    /** @var \Drupal\tmgmt\Entity\Job $job */
    $job = reset($job_items)->getJob();
    foreach ($job_items as $job_item) {
      if ($job->isContinuous()) {
        $job_item->active();
      }
      // Pull the source data array through the job and flatten it.
      $data = $this->dataHelper->filterTranslatable($job_item->getData());
      $translation = [];
      // Build OpenAI q param and preserve initial array keys.
      foreach ($data as $key => $value) {
        // Get translation from OpenAI.
        $result = $this->openAiRequestTranslation($job, $value['#text']);
        if (!empty($result->choices[0])) {
          $choice = $result->choices[0]->toArray();
          $translation[$key]['#text'] = Html::decodeEntities(trim($choice['text']));
        }
      }

      try {
        // Save the translated data through the job.
        // Only reached when all translation requests have succeeded.
        $job_item->addTranslatedData($this->dataHelper
            ->unflatten($translation));
      }
      catch (TMGMTException $e) {
        $job->rejected('Translation has been rejected with following error: @error',
          array('@error' => $e->getMessage()), 'error');
      }
    }
  }

  /**
   * Set credentials.
   */
  public function openAiClient($translator) {
    try {
      if ($translator->getSetting('openai_api_service') == 'azure_openai') {
        $this->client = \OpenAI::factory()
          ->withBaseUri($translator->getSetting('openai_api_base_url'))
          ->withHttpHeader('api-key', $translator->getSetting('openai_api_key'))
          ->withQueryParam('api-version', $translator->getSetting('openai_api_version'))
          ->make();
      }
      else {
        $org = !empty($translator->getSetting('openai_api_org')) ? $translator->getSetting('openai_api_org') : NULL;
        if (!empty($translator->getSetting('openai_api_key'))) {
          $this->client = \OpenAI::client($translator->getSetting('openai_api_key'), $org);
        }
        else {
          \Drupal::messenger()->addMessage('Missing or incorrect OpenAI API Key', 'error');
        }
      }
    }
    catch (\Exception $ex) {
      \Drupal::messenger()->addMessage($ex->getMessage(), 'error');
    }
    return $this;
  }

  /**
   * Get models list.
   */
  public function getModels() {
    $list = [];
    try {
      $response = $this->client->models()->list();
      if (!empty($response->data)) {
        foreach ($response->data as $item) {
          $list[$item->id] = $item->id;
        }
      }
    }
    catch (\Exception $ex) {
      \Drupal::messenger()->addMessage($ex->getMessage(), 'error');
    }
    return $list;
  }

  /**
   * Creates a completion request for the provided prompt and parameters.
   */
  public function completionsRequest($configuration) {
    try {
      $response = $this->client->completions()->create($configuration);
      return $response;
    }
    catch (\Exception $ex) {
      \Drupal::messenger()->addMessage($ex->getMessage(), 'error');
    }
  }

}
